#include <ntddk.h>

ULONG64 _MyCpuid();
ULONG64 _ReadMsr(ULONG32 reg);
ULONG64 _Cr4();

typedef struct _CR4_REG {
    unsigned VME        :1;            // Virtual Mode Extensions
    unsigned PVI        :1;            // Protected-Mode Virtual Interrupts
    unsigned TSD        :1;            // Time Stamp Disable
    unsigned DE         :1;            // Debugging Extensions
    unsigned PSE        :1;            // Page Size Extensions
    unsigned PAE        :1;            // Physical Address Extension
    unsigned MCE        :1;            // Machine-Check Enable
    unsigned PGE        :1;            // Page Global Enable
    unsigned PCE        :1;            // Performance-Monitoring Counter Enable
    unsigned OSFXSR     :1;            // OS Support for FXSAVE/FXRSTOR
    unsigned OSXMMEXCPT :1;            // OS Support for Unmasked SIMD Floating-Point Exceptions
    unsigned Reserved1  :2;            //
    unsigned VMXE       :1;            // Virtual Machine Extensions Enabled
    unsigned Reserved2  :18;           //
} CR4_REG, *PCR4_REG;

ULONG64  msr0x480[16];	

const char *legend[ 12 ] = {
			"IA32_VMX_BASIC_MSR",		// 0x480
			"IA32_VMX_PINBASED_CTLS_MSR",	// 0x481
			"IA32_VMX_PROCBASED_CTLS_MSR",	// 0x482
			"IA32_VMX_EXIT_CTLS_MSR",	// 0x483
			"IA32_VMX_ENTRY_CTLS_MSR",	// 0x484
			"IA32_VMX_MISC_MSR",		// 0x485
			"IA32_VMX_CR0_FIXED0_MSR",	// 0x486
			"IA32_VMX_CR0_FIXED1_MSR",	// 0x487
			"IA32_VMX_CR4_FIXED0_MSR",	// 0x488
			"IA32_VMX_CR4_FIXED1_MSR",	// 0x489
			"IA32_VMX_VMCS_ENUM_MSR"	// 0x48A
			};

enum { legend_count = 11 };

void DriverUnload(PDRIVER_OBJECT pDriverObject)
{
	DbgPrint("Driver unloading\n");
}

NTSTATUS DriverEntry(PDRIVER_OBJECT DriverObject, PUNICODE_STRING RegistryPath) 
{
	ULONG64 ret;
	ULONG32 msr;
	ULONG64 cr4;
	PCR4_REG pCr4;
	char buff[16];
	const char** msrStr = legend;
	int ind = 0;

	RtlZeroMemory(buff, sizeof(buff));
	DriverObject->DriverUnload = DriverUnload;
	ret = _MyCpuid();
	RtlCopyMemory(buff, (void*)ret, sizeof(buff));
	DbgPrint("ret :%p\n", ret);
	DbgPrint("Hello World! cpu:%s\n", buff);


	for(ind=0; ind < legend_count; ++ind) {
		msr = 0x480 + ind;
		DbgPrint("msr: %#010x  %30s	=> %#018llx\n", msr, *(msrStr++), _ReadMsr(msr));
	}

	cr4 = _Cr4();
	pCr4 = (PCR4_REG) &cr4;
	DbgPrint("CR4: %#018llx ... CR4.VMXE=%d\n", cr4, pCr4->VMXE);

	return STATUS_SUCCESS;

}

